#include "basic_creatures.h"
#include "Object.h"
#include "Dungeon.h"

#include <stdio.h>
#include <Alert.h>

// PLAYER

Player::Player( )
:	Creature("Player")
{
	info.INT.base = (random() % 3) + 3; info.INT.curr = info.INT.base;
	info.STR.base = (random() % 3) + 3; info.STR.curr = info.STR.base;
	info.HP.base = 20 + random() % 20; info.HP.curr = info.HP.base;

	skills.Add( new Skill("Hand to hand",50) );
	skills.Add( new Skill("1H Sword",50) );
}

Player::~Player()
{
}

void
Player::Attack( Creature * target )
{
	char text[256];
	
	Object * weapon = FindEquiped( RIGHT_HAND );
	
	if ( weapon )
	{ // attack with weapon
		if ( TestSkill( weapon->skill ) )
		{ // hit
			weapon->Use( target, this );
		} else
		{ // miss
			sprintf(text,"%s misses %s.",name,target->name);
			game_message( info.pos, text );
		}
	
	} else
	{ // attack bare hand
		if ( TestSkill("Hand to hand") )
		{
			damage dmg;
			dmg.source = this;
			dmg.amount = random() % info.STR.curr + 1;
		
			sprintf(text,"%s punches %s for %d dmg.",name,target->name,dmg.amount);
			game_message( info.pos, text );

			target->Damage( dmg );
		} else
		{
			sprintf(text,"%s misses %s.",name,target->name);
			game_message( info.pos, text );
		}
	}

}

PlayerInfo::PlayerInfo()
{
	bmp[0].file = "/add-ons/gfx/warrior/warrior";
	bmp[0].id = 'PLwa';

	bmp[1].file = NULL;
	bitmaps = bmp;
	name = "Warrior";
	
	flags = PLAYABLE;
}

Creature *
PlayerInfo::Create( )
{
	Player * player = new Player();
	strcpy(player->info.bitmap, "add-ons/gfx/warrior/warrior");
	player->internal_name = name;
	
	return player;
}

// ORC

Orc::Orc()
:	Creature("orc")
{
	info.INT.base = 4; info.INT.curr = 4;
	info.STR.base = 7; info.STR.curr = 7;
	info.HP.base = 6; info.HP.curr = 6;

	skills.Add( new Skill("Sword", 40) );

	if ( (random() % 5) == 0 )
	{ // every fifth orc has a health potion (approx)
		Object * obj = current_dungeon->CreateObject("potion_health");
		if (obj)
		{
			inventory.Add( obj->id );
		}
	}
}

Orc::~Orc()
{
}

void
Orc::AI()
{
	Creature * player = current_dungeon->FindCreature(1);
	if (player && _dist(info.pos,player->info.pos) < 8 && current_dungeon->CastRay(info.pos, player->info.pos, 0) )
	{ // player in sight [max sight 8 tiles], go for it!
		if ( FindPath( player->info.pos ) )
			FollowPath();
	} else
	{ // player not in sight
		if ( HasPath() )
		{ // we have a path, follow it!
			FollowPath();
		} else
		{ // wander aimlessly
			int new_x = info.pos.x + (random() % 3) - 1;
			int new_y = info.pos.y + (random() % 3) - 1;
			MoveTo( Pos(new_x,new_y,info.pos.level) );
		}
	}
}

void
Orc::Death()
{
	Creature::Death();
}

void
Orc::Attack( Creature * target )
{
	char text[256];

	if ( strcmp( target->name, name ) == 0 )
		return; // don't attack fellow orcs
		
	if ( TestSkill("Sword") )
	{
		damage dmg;
		dmg.source = this;
		dmg.amount = random() % 4 + 3;
		
		sprintf(text,"%s hits %s for %d dmg.",name,target->name,dmg.amount);
		game_message( info.pos, text );
		target->Damage( dmg );
	} else {
		sprintf(text,"%s misses %s.",name,target->name);
		game_message( info.pos, text );
	}
}

OrcInfo::OrcInfo()
{
	bmp[0].file = "/add-ons/gfx/orc/orc";
	bmp[0].id = 'ORwa';

	bmp[1].file = NULL;
	bitmaps = bmp;
	name = "orc_basic";
}

Creature *
OrcInfo::Create()
{
	Orc * orc = new Orc();
	strcpy(orc->info.bitmap, "add-ons/gfx/orc/orc");
	orc->internal_name = name;

	return orc;
}

bool
OrcInfo::ShouldPlace( int level )
{
	return (random() % 3) == 0;
}

// RAT

Rat::Rat()
:	Creature("rat"),
	irritated(false)
{
	info.INT.base = 2; info.INT.curr = 2;
	info.STR.base = 4; info.STR.curr = 4;
	info.HP.base = 4; info.HP.curr = 4;

	skills.Add( new Skill("Claw", 50) );
}

Rat::~Rat()
{
}

void
Rat::AI()
{
	if ( !irritated )
	{ // wander aimlessly
		int new_x = info.pos.x + (random() % 3) - 1;
		int new_y = info.pos.y + (random() % 3) - 1;
		current_dungeon->MoveCreature( info.id, info.pos, Pos(new_x,new_y,info.pos.level) );
		return;
	}

	Creature * trg = current_dungeon->FindCreature(target);
	if (trg && current_dungeon->CastRay(info.pos, trg->info.pos, 0) )
	{ // target in sight, go for it!
		if ( FindPath( trg->info.pos ) )
			FollowPath();
	} else
	{ // target not in sight
		if ( HasPath() )
		{ // we have a path, follow it!
			FollowPath();
		} else
		{ // wander aimlessly
			int new_x = info.pos.x + (random() % 3) - 1;
			int new_y = info.pos.y + (random() % 3) - 1;
			current_dungeon->MoveCreature( info.id, info.pos, Pos(new_x,new_y,info.pos.level) );
		}
	}
}

void
Rat::Death()
{
	Creature::Death();
}

void
Rat::Attack( Creature * target )
{
	char text[256];

	if ( strcmp( target->name, name ) == 0 )
		return; // don't attack fellow orcs
		
	if ( TestSkill("Claw") )
	{
		damage dmg;
		dmg.source = this;
		dmg.amount = random() % 3 + 1;
		
		sprintf(text,"%s claws %s for %d dmg.",name,target->name,dmg.amount);
		game_message( info.pos, text );
		target->Damage( dmg );
	} else {
		sprintf(text,"%s misses %s.",name,target->name);
		game_message( info.pos, text );
	}
}

void
Rat::Damage( damage dmg )
{
	if ( !irritated )
	{
		char text[256];
		sprintf(text, "rat turns to attack %s!", dmg.source->name);
		game_message( info.pos, text, TextMode::BOLD, (rgb_color){255,200,0,255} );
	}
	
	irritated = true;
	target = dmg.source->info.id;
	
	Creature::Damage( dmg );
}

RatInfo::RatInfo()
{
	bmp[0].file = "/add-ons/gfx/white_rat/white_rat";
	bmp[0].id = 'RAwh';

	bmp[1].file = NULL;
	bitmaps = bmp;
	name = "rat_white";
}

Creature *
RatInfo::Create()
{
	Rat * rat = new Rat();
	strcpy(rat->info.bitmap, "add-ons/gfx/white_rat/white_rat");
	rat->internal_name = name;
	
	return rat;
}

bool
RatInfo::ShouldPlace( int level )
{
	return true;
}

// BASIC BUILDER

BasicBuilder::BasicBuilder()
:	CreatureBuilder( "BasicBuilder" )
{
	info_original[0] = &player_info;
	info_original[1] = &orc_info;
	info_original[2] = &rat_info;
	info_original[3] = NULL;
	info = info_original;

	text = "BasicBuilder";
}

Creature *
BasicBuilder::Create( const char * name )
{
	return NULL;
}

uint32
BasicBuilder::AddonVersion()
{
	return CREATURE_ADDON_VERSION;
}

// EXPORT

CreatureBuilder *
CreateBuilder()
{
	return new BasicBuilder;
}
