#ifndef CREATURE_ADDON_H
#define CREATURE_ADDON_H

#include "Creature.h"
#include "Graphics.h"

enum {
	PLAYABLE = 0x00000001 // player can play this creature type
};

class Dungeon;

class CreatureInfo
{
	public:
		CreatureInfo();

		status_t LoadGraphics( Graphics * ); // load bitmap(s)
		virtual Creature * Create( );
		virtual bool ShouldPlace( int level );
		
		const char		* name;
		
		uint32			flags;
		
	protected:
		bitmap_info		* bitmaps;
};

class CreatureBuilder
{
	public:
		CreatureBuilder( const char * );

		virtual uint32 AddonVersion()=0;

		virtual Creature * Create( const char * name = NULL );

		int NumCreatures();
		CreatureInfo * CreatureAt( int );
		
		status_t LoadGraphics( Graphics * );
		
		const char * text;
		
	protected:
		CreatureInfo ** info;
};

class CreatureInfoSet
{
	public:
		CreatureInfoSet();
		~CreatureInfoSet();
		
		void LoadAddons();
		void UnloadAddons();
		
		int NumCreatures();
		CreatureInfo * CreatureAt( int );
		CreatureInfo * FindCreature( const char * );
		status_t LoadGraphics( Graphics * );
		
	protected:
		List<CreatureInfo>		info_list;
		List<CreatureBuilder>	builder_list;
		List<image_id>			addon_list;
};

extern "C" __declspec(dllexport) CreatureBuilder * CreateBuilder();

const uint32 CREATURE_ADDON_VERSION = 0x0101;

#endif
