#ifndef OBJECT_ADDON_H
#define OBJECT_ADDON_H

#include "Object.h"
#include "Graphics.h"
#include "misc.h"

class Dungeon;

class ObjectInfo
{
	public:
		ObjectInfo();

		status_t LoadGraphics( Graphics * ); // load bitmap(s)
		virtual Object * Create();
		virtual bool ShouldPlace( int level );
				
		const char		* name;
		bitmap_info		* bitmaps;
		uint32			flags;

	protected:
		uint32	bitmap_id;
};

class ObjectBuilder
{
	public:
		ObjectBuilder( const char * );

		uint32 AddonVersion();
		
		virtual Object * Create( const char * name = NULL );

		int NumObjects();
		ObjectInfo * ObjectAt( int );

		status_t LoadGraphics( Graphics * );
		
		const char * text;

	protected:
		ObjectInfo ** info;
};

extern "C" __declspec(dllexport) ObjectBuilder * CreateObjectBuilder();

class ObjectInfoSet
{
	public:
		ObjectInfoSet();
		~ObjectInfoSet();
		
		void LoadAddons();
		void UnloadAddons();
		
		int NumObjects();
		ObjectInfo * ObjectAt( int );
		ObjectInfo * FindObject( const char * );
		status_t LoadGraphics( Graphics * );
		
		status_t	Load( BFile * );
		status_t	Save( BFile * );
		
	protected:
		List<ObjectInfo>		info_list;
		List<ObjectBuilder>		builder_list;
		List<image_id>			addon_list;
};

#define OBJECT_ADDON_VERSION 0x0101

#endif
